/** \file combinatorial_map.hh
    \brief File for the combinatorial maps */

#ifndef __COMBINATORIAL_MAP__
#define __COMBINATORIAL_MAP__

#include <point.hh>
#include <tab_sym_bool.hh>
#include "level_set.hh"
#include "image.hh"
#include "combi_map_gen.hh"


/** Class for the combinatorial maps */
class combinatorial_map: protected combi_map_gen
{
typedef enum {cm_draw_single,cm_draw_both,cm_draw_all_single,cm_draw_all_both} draw_type;
friend class pyramid;
private:
  /// level of the map in the pyramyd
  int level;
  // An array to store the orientation of the darts
  tab_sym_bool open_parenthesis;

public:


  /// Constructor
  combinatorial_map():combi_map_gen(NULL),level(0)
  {}
  /// Constructor
  combinatorial_map(pyramid &P):combi_map_gen(&P),level(0)
  {}
  /// Virtual destructor
  virtual ~combinatorial_map()
  {}
  /// Function to access to the sigma of one dart of the combinatorial map
  dart &sigma(dart d)
  {
    return sig[d];
  }
  /// Function to access to the alpha of one dart of the combinatorial map
  dart &alpha(dart d)
  {
    return alph[d];
  }
  /// Function to read the vertex of one dart of the combinatorial map
  vertex mu(dart d) const
  {
    return m(d);
  }
  /// Function to read one dart of one vertex of the combinatorial map
  dart mu_1(vertex v) const
  {
    return m_1(v);
  }
  /// Function that return true if the dart is an openparenthesis for an inclused area
  bool is_open_parenthesis(dart d) const
  {
    return open_parenthesis(d);
  }
  /// give the level of the combinatorial map
  dart give_level()
  {
    return level;
  }
  /// give the background of the combinatorial map
  unsigned int give_background()
  {
    return background;
  }

  /// Give the max level of the dart
  int give_level(dart d);

  /// Give the width of the base map
  int give_width();
  /// Give the height of the base map
  int give_height();
  /// Give the alpha of the dart in the base map
  dart give_alpha_base(dart d);
  /// Give the sigma of the dart in the base map
  dart give_sigma_base(dart d);
  /// Give the departure point of the dart
  void give_point(dart d,point &p);
  /// Give the departure pixel of the dart
  void give_pixel(dart d,point &p);
  /// Give the orientation of the dart
  move_type give_move(dart d);

  /// Give an iterator positioned on the first element of the vertex list
  vertex_iterator &give_vertex_first();
  /// Give an iterator positioned on the last element of the vertex list
  vertex_iterator &give_vertex_last();
  /// Give an iterator positioned on the element after the last of the vertex list
  vertex_iterator &give_vertex_end();

  /// Give an iterator positionned on the first dart of the level_set for the given level
  dart_iterator &give_dart_first();
  /// Give an iterator positionned on the first dart of the given level
  dart_iterator &give_dart_begin();
  /// Give an iterator positionned after the last dart of the given level (loop : for(i=level_set.first();i<level.end();i++)
  dart_iterator &give_dart_last();
  /// Give an iterator positionned on the last dart of the given level (loop : for(i=level_set.first();i<=level.last();i++)
  dart_iterator &give_dart_end();

  /// Function that put in the two lists the neighbors for the first list and the inclused area in the second
  void give_all_neighbors(vertex,list_label&,list_label&);
  /// Function that put in the list the neighbors
  void give_neighbors(vertex,list_label&);
  /// Function that put in the list the included neighbors
  void give_included(vertex,list_label&);
  /// Function that print on the standard output the list of the vertex and the sigma permutation
  void display();
  /// Function to draw the border on the picture
  void draw_border(image&,unsigned char*,draw_type=cm_draw_single);
  /// Function to draw the border of one area on the picture
  void draw_vertex_border(vertex,image&,unsigned char*,draw_type=cm_draw_single);
  /// Return a dart that is connected to a non included neighbor
  dart give_neighbor_dart(vertex);
  
  
  /** Class that manage the border of a region on the base map */
  class segment{
  private:
    // Pointor on the pyramid of the segment
    pyramid *pyr;
    // Level of the map of the segment
    int level;
    // First dart of the segment
    dart b_first;
    // Last dart of the segment
    dart b_last;
  public:
    /// Constructor
    segment();
    /// Constructor
    segment(dart,dart,int,pyramid *);

    typedef enum {first_dart,last_dart,end_dart} position;
    class iterator
    {
      private:
    	// Pointor on the curent segment
    	segment *seg;
    	// Current dart of the segment
    	dart b_current;
      public:
    	/// Default constructor
    	iterator():seg(NULL), b_current(0)
    	{}
    	/// Constructor
    	iterator(segment *,position p=first_dart);
    	/// Operator =
    	iterator &operator=(const iterator &);
    	/// Accessor to the element
    	dart operator *()
    	{
    	  return b_current;
    	}
    	/// Post operator ++
    	iterator &operator++();
    	/// Pre operator ++
    	iterator &operator++(int);
    	/// Post operator --
    	iterator &operator--();
    	/// Pre operator --
    	iterator &operator--(int);
    	/// Fonction that give the movement of the dart
    	move_type give_move();
    	/// Fonction that give the departure point of the dart
    	void give_point(point &);
    	/// Fonction that give the two surrounding pixel of the dart, if one is outside, the coordinates were (-1,-1)
	/// return false if on of the pixel is outside of the initial picture
	/// The first on is the pixel on the left of the dart
    	bool give_pixel(point &,point &);

    	/// Bool operator ==
    	bool operator==(const iterator &) const;
    	/// Bool operator !=
    	bool operator!=(const iterator &) const;
    };

    /// first dart of the border
    iterator & first();
    /// after the last dart of the border (loop : for(i=segment.first();i<segment.end();i++)
    iterator & end();
    /// last dart of the border (loop : for(i=segment.first();i<=segment.last();i++)
    iterator & last();
  };

  /// Function that give a segment corresponding to the dart
  segment give_segment(dart d)
  {
    return segment(d,alph(d),level,pyr);
  }
};


typedef combinatorial_map::segment segment;
typedef combinatorial_map::segment::iterator segment_iterator;

#endif // __COMBINATORIAL_MAP__
